#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  untitled.py
#  
#  Copyright 2016 eddiewrc <eddiewrc@eddiewrc-Precision-Tower-7810>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  



def main():
	
	return 0

if __name__ == '__main__':
	main()




# -*- coding: utf-8 -*-
import sysconfig, os
from b2bTools.singleSeq.PSPer.phase_transition_hmm import phase_hmm
import string
import numpy as np


def leggifasta(database):
		f=open(database)
		uniprot=f.readlines()
		f.close()
		dizio={}
		for i in uniprot:
			if i[0]=='>':
					uniprotid=i.strip('>\n')
					dizio[uniprotid]=''
			else:
				dizio[uniprotid]=dizio[uniprotid]+i.strip('\n').upper()
		return dizio
def standalone(input_obj):
	features='dyna_back,psipred,ef,dyna_side'
	window=4
	verbose=0
	def check_sequences(seqs):
		for i in list(seqs.keys()):
			if i=='extra_predictions':
				continue
			if seqs[i].strip()=='SEMBRI QUEL FESSO DI MIO ZIO VITO':
				return {'error':'tua zia troverebbe il paragone alquanto ardito'}
			if i=='extra_predictions':  # questa cosa è un pochino grezza, ma noi siamo grezzi nell'anima
				continue
			if not seqs[i].isalpha():
				return {'error':'invalid char in sequence '+i}
			if len(seqs[i])>3000:
				#print len(seqs[i])
				return {'error':'sequence '+i+' too long, maximum lenght is 3000 amino acids'}
			if len(seqs[i])<20:
				#print len(seqs[i])
				return {'error':'sequence '+i+' short, minimum lenght is 20 amino acids'}
		return True
	def clean_psipred_tmp():
		scriptDir = sysconfig.get_paths()["purelib"]+'/b2bTools/singleSeq/DisoMine'
		print('Running Cleaner')
		TEMPDIR = scriptDir+'/vector_builder/psipred/tmp/'
		for filename in os.listdir(TEMPDIR):
		    file_path = os.path.join(TEMPDIR, filename)
		    try:
		        if os.path.isfile(file_path) or os.path.islink(file_path):
		            os.unlink(file_path)
		        elif os.path.isdir(file_path):
		            shutil.rmtree(file_path)
		    except Exception as e:
		        print('Failed to delete %s. Reason: %s' % (file_path, e))

	def load_model():
		mod=phase_hmm()
		mod.fit()
		scaler=None
		return mod,scaler


	def format_output(disorder,viterbi,seqs,features):
		out=[]

		for i in range(len(disorder.keys())):
			assert len(features[list(disorder.keys())[i]][:,i])==len(features[list(disorder.keys())[i]][:,0])
			entry={}
			features[list(disorder.keys())[i]]=np.array(features[list(disorder.keys())[i]])
			entry['proteinID']=list(disorder.keys())[i]
			entry['sequence']=seqs[list(disorder.keys())[i]]
			entry['protein_score']=disorder[list(disorder.keys())[i]]
			entry['viterbi']=viterbi[list(disorder.keys())[i]]
			entry['complexity']=list(features[list(disorder.keys())[i]][:,0])
			entry['arg']=list(features[list(disorder.keys())[i]][:,1])
			entry['tyr']=list(features[list(disorder.keys())[i]][:,2])
			entry['RRM']=list(features[list(disorder.keys())[i]][:,3])
			entry['disorder']=list(features[list(disorder.keys())[i]][:,4])
			out+=[entry]
		return {'results':out}



	def predict_fasta(fil,crunch=100):
		if type(fil)==str:
			try:
				a=leggifasta(fil)
			except:
				return {'error':"problems in the fasta file"}
		elif type(fil)==dict:
			a=fil
		else:
			return {'error':'internal error, wrong object passed to the standalone, it must be a dict or a string'}
		check=check_sequences(a)
		if check!=True:
			return check
		targets=list(a.keys())[:]
		results_dict={}
		cont=0
		dyna={}
		side={}
		ef={}
		v=model.build_vector(a)
		results_dict=model.predict_proba(v)
		viterbi=model.viterbi(v)
		fea={}
		printable=string.printable
		for id in v.keys():
			vet=[]
			for i in v[id]:
				t=[]
				cont=0
				for j in i:
					if cont==1:
						l=printable.index(j)
						t+=[float(l)]
					elif cont==2:
						l=printable.index(j)
						t+=[float(l)]
					else:
						l=printable.index(j)
						t+=[float(l)]
						cont+=1
				vet+=[t]
			fea[id]=np.array(vet)

		results=format_output(results_dict,viterbi,a,fea)
		return results
	# This should not be necessary with new disomine setup, commented out
	#clean_psipred_tmp()
	model,scaler=load_model()
	results=predict_fasta(input_obj)
	return results

def main(args):
	#print standalone("example.fasta")
	a=leggifasta('input_files_examples/example_toy.fasta')
	a['extra_predictions']=False
	print(standalone(a))
	#from memory_profiler import memory_usage
	#mem_usage = memory_usage(standalone,interval=0.01)
	#print('Memory usage (in chunks of .1 seconds): %s' % mem_usage)
	#print('Maximum memory usage: %s' % max(mem_usage))
	#cProfile.run('standalone("example.fasta")')

if __name__ == '__main__':
	import sys
	sys.exit(main(sys.argv))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  hmmsearch_otf.py
#
#  Copyright 2018 scimmia <scimmia@scimmia-ThinkPad-L540>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  parseHmmer.py
#
#  Copyright 2018 Daniele Raimondi <daniele.raimondi@vub.be>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#

def parseBoundaries(line):
	# qwwe  42 IFVGQLDKETTREELNRRFSTHGKIQDINLIFK---PTNIFAFIKYETEEAAAAALESENHAIFLNKTMH 108
	tmp = line.strip().split()
	return tmp[1], tmp[-1]

def parsePosteriori(line):
	tmp = line.strip().split()
	r = []
	for i in tmp[0]:
		p = castPosterior(i)
		if p == -1:
			continue
		r.append(p)
	return r

def castPosterior(v):
	if v == "*":
		return 10
	elif v == ".":
		return -1
	return int(v)

def parseOut(f):
	ifp = open(f)
	lines = ifp.readlines()
	alscores = []
	domains = []
	length = -1
	l = 0
	while l < len(lines):

		if "Scores for complete sequences" in lines[l]:
			if "[No hits" in lines[l+5]:
				return None, None
			else:
				tmp = lines[l+4].strip().split()
				#print tmp
				bitscore = float(tmp[1])
				evalue = float(tmp[3])
				alscores.append(( bitscore, evalue))
		if "Domain annotation for each sequence" in lines[l]:
			while l < len(lines):
				if "== domain" in lines[l]:
					start, end = parseBoundaries(lines[l+3])
					domains.append((int(start), int(end), parsePosteriori(lines[l+4])))
					l+=4
				l+=1
				if "Internal pipeline statistics summary:" in lines[l]:
					break
		if "residues searched)" in lines[l]:
			tmp = lines[l].strip().split()
			#print tmp
			length = int(tmp[3][1:])

		l += 1
	return domains, length

def getFeatsFromHmmer(v,l):

	if v==None:
		return None
	r = [0]*l
	for i in v:
		start = i[0]
		end = i[1]
		j = start-1
		while j < end:
			a=i[2].pop(0)
			r[j] = a
			j+=1
	return r

def main():
	a, l =  parseOut("output")

	#print a, l
	#raw_input()
	b = getFeatsFromHmmer(a,l)
	#print b
	c=hmmer_wrapper()
	c.fit_hmmer()
	print (c.predict({'a':'AAAAAAAAAAAACCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA','B':'VYVGNLGNNGNKTELERAFGYYGPLRSVWVARNPPGFAFVEFEDPRDAADAVRELDGRTLCGCRVR'}))
import os

class hmmer_wrapper:
	def __init__(self,root=''):

		from b2bTools.singleSeq.PSPer.Constants import hmmbuild_bin, hmmscan_bin

		self.ali_file=root+'rrm_align.pfam'
		self.hmmbuild_bin=hmmbuild_bin
		self.hmmscan_bin=hmmscan_bin
		self.root=root
	def fit_hmmer(self):
		#print self.hmmbuild_bin+' '+'phase_trans.hmm'+' '+self.ali_file +' > /dev/null'
		os.system(self.hmmbuild_bin+' '+self.root+'phase_trans.hmm'+' '+self.ali_file +' > /dev/null')
	def predict(self,seqs):
		resu={}
		for i in seqs.keys():
			f=open(self.root+'tmp.fasta','w')
			f.write('>tmp\n'+str(seqs[i])+'\n')
			f.close()
			os.system(self.hmmscan_bin+' -o '+self.root+'out_hmmer.tmp '+self.root+'phase_trans.hmm '+ self.root+'tmp.fasta')
			#print self.hmmscan_bin+' -o '+self.root+'out_hmmer.tmp '+self.root+'phase_trans.hmm '+ self.root+'tmp.fasta'
			a, l =  parseOut(self.root+'out_hmmer.tmp')
			b = getFeatsFromHmmer(a,l)
			os.system('rm '+self.root+'out_hmmer.tmp')
			os.system('rm '+self.root+'tmp.fasta')
			if b==None:
				b=[0]*len(str(seqs[i]))
			resu[i]=b
		return resu



if __name__ == '__main__':
	a=hmmer_wrapper()
	print (a.predict({'a':'AAAAAAAAAAAACCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA','B':'VYVGNLGNNGNKTELERAFGYYGPLRSVWVARNPPGFAFVEFEDPRDAADAVRELDGRTLCGCRVR'}))



# -*- coding: utf-8 -*-
import os
import tempfile
import subprocess, sys

cwd = os.path.dirname(os.path.abspath(__file__))

DATADIR = cwd + '/psipred/data/'
TEMPDIR = cwd + '/psipred/tmp/'

if sys.platform == 'linux':
    SEQ2MTX = cwd + '/psipred/bin/linux/seq2mtx'
    PSIPRED = cwd + '/psipred/bin/linux/psipred'
    PSIPASS2 = cwd + '/psipred/bin/linux/psipass2'
elif sys.platform == 'darwin': # MacOS hosts
    SEQ2MTX = cwd + '/psipred/bin/osx/seq2mtx'
    PSIPRED = cwd + '/psipred/bin/osx/psipred'
    PSIPASS2 = cwd + '/psipred/bin/osx/psipass2'

for exec_path in [SEQ2MTX, PSIPRED, PSIPASS2]:
    if os.path.exists(exec_path):
        os.chmod(exec_path, int('777', 8))

def run(input):
    fasta = os.path.basename(input)
    name = fasta.split('.')[-2]

    print('Psipred Running... processing: ' + name)

    with open(TEMPDIR + name + ".mtx", "w") as mtx_file:
        seq2mtx = subprocess.run([SEQ2MTX, input], stdout=mtx_file)

    with open(TEMPDIR + name + ".ss", "w") as ss_file:
        psipred = subprocess.run( [PSIPRED, TEMPDIR+name+".mtx", DATADIR+'weights.dat', DATADIR+'weights.dat2', DATADIR+'weights.dat3'], stdout=ss_file)

    with open(TEMPDIR + name + ".horiz", "w") as horiz_file:
        psipass2 = subprocess.run( [PSIPASS2, DATADIR+'weights_p2.dat', '1', '1.0', '1.0', TEMPDIR+name+".ss2", TEMPDIR+name+".ss"], stdout=horiz_file)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import setuptools

import os
parent_path = os.path.relpath('.')
path_to_file = parent_path+'/README.md'
with open(path_to_file, 'r') as f:
    long_description = f.read()

setuptools.setup(
    name="b2bTools",
    version="3.0.1b13",
    author="Wim Vranken",
    author_email="Wim.Vranken@vub.be",
    description="bio2Byte software suite to predict protein biophysical properties from their amino-acid sequences",
    license="OSI Approved :: GNU General Public License v3 (GPLv3)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    maintainer="Jose Gavalda-Garcia, Adrian Diaz, Wim Vranken",
    maintainer_email="jose.gavalda.garcia@vub.be, adrian.diaz@vub.be, wim.vranken@vub.be",
    url="https://bio2byte.be",
    project_urls={
        # "Bug Tracker": "https://bitbucket.org/bio2byte/b2btools/issues",
        "Documentation": "https://bio2byte.be/b2btools/package-documentation",
        # "Source": "https://bitbucket.org/bio2byte/b2btools/",
        "HTML interface" : "https://bio2byte.be/b2btools"
    },
    packages=setuptools.find_packages(exclude=("**/test/**",)),
    include_package_data=True,
    keywords="b2bTools,biology,bioinformatics,bio-informatics,fasta,proteins,protein-folding",
    classifiers=[
        "Natural Language :: English",
        # "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: MacOS",
        "Operating System :: POSIX :: Linux",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Scientific/Engineering :: Physics",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Education",
        "Development Status :: 5 - Production/Stable"
    ],

    python_requires='>=3.6.*, <3.10 ',

    # install_requires=[
    #     'future==0.18.2',
    #     'joblib==1.0.1',
    #     'networkx==2.6.2',
    #     'numpy==1.21.2',
    #     'Pillow==8.3.1',
    #     'pomegranate==0.14.4',
    #     'PyYAML==5.4.1',
    #     # 'scikit-learn==0.20.4',
    #     'scikit-learn>=1.0.1',
    #     'scipy==1.7.1',
    #     'six==1.16.0',
    #     'threadpoolctl==2.2.0',
    #     'torch==1.2.0',
    #     'torchvision==0.4.0'
    # ]

    install_requires=[
        # 'future==0.18.2',
        'joblib',
        'networkx',
        'pomegranate',
        # 'numpy',
        'numpy<=0.19.5',
        'Pillow',
        'PyYAML',
        # 'scikit-learn>=1.0.1',
        'scikit-learn>=0.23.1',
        'scipy',
        'six',
        'threadpoolctl',
        'torch>=1.8.0',
        'torchvision'
    ]
)



from ..singleSeq.Predictor import MineSuite
from ..singleSeq import constants


class SingleSeq:
    """
    Stores the sequences, the filename where they were read from, tools to be executed and the result of the
    predicitons.

    Parameters:

            fileName (str): Path to the fasta file where the sequences of interest are located.
    """

    def __init__(self, fileName):
        """
        Constructor for the SingleSeq class. Reads the fasta file that contains the sequences, calls a function that
        reads the sequences and creates the structure to store all the information required to run and store the
        predicitons.

        Parameters:

            fileName (str): Path to the fasta file where the sequences of interest are located.


        """
        self.__mineSuite__ = MineSuite()
        self.__sequenceFileName__ = fileName
        self.__sequences__ = self.__mineSuite__.readFasta(fileName)
        self.__predictor_runners__ = {
            constants.TOOL_DYNAMINE: self.__mineSuite__.dynaMine,
            constants.TOOL_DISOMINE: self.__mineSuite__.disoMine,
            constants.TOOL_EFOLDMINE: self.__mineSuite__.eFoldMine,
            constants.TOOL_AGMATA: self.__mineSuite__.agmata,
        }

        self.__results__ = {}

    # This method has the responsibility of running the asked predictions in a smart way: it omits the tools that already have a result
    def predict(self, tools=[]):
        """
        This method has the responsibility of running the asked predictions in a smart way: it omits the tools that
        already have a result and runs the remaining.

        Parameters:

            tools (list[str]) : List of tools to be ran. Any combination of the following tools is accepted:
                - "dynamine"
                - "disomine"
                - "efoldmine"
                - "agmata"

        Returns:

            obj: self
        """

        for tool in tools:
            dependencies = constants.DEPENDENCIES[tool]

            if (len(dependencies) > 0):
                self.predict(tools=dependencies)

            if not self.__results__.get(tool):

                predictor_function = self.__predictor_runners__[tool]
                predictor_function.allPredictions = self._all_predictions()

                if tool == constants.TOOL_EFOLDMINE:
                    predictor_function.predictSeqs(self.__sequences__, dynaMinePreds=predictor_function.allPredictions)
                else:
                    predictor_function.predictSeqs(self.__sequences__)

                self.__results__[tool] = predictor_function.allPredictions
                print("executed {0}".format(tool))

        return self

    # Instead of using a list of tools as in predict method, this one uses flags to create a list of tools and call
    # the predict method:
    def semantic_predict(
            self,
            dynamics=False,
            aggregation=False,
            early_folding_propensity=False,
            disorder=False,
    ):
        """
        Instead of using a list of tools as in predict method, this one uses flags on the (high level) type of
        predictions desired to create a list of tools and call the predict method.

        Parameters:

            dynamics (bool, optional) : Whether or not dynamics predictions are to be ran
            aggregation (bool, optional) : Whether or not aggregation predictions are to be ran
            early_folding_propensity (bool, optional) : Whether or not early folding propensity predictions are to be ran
            disorder (bool, optional) : Whether or not disorder predictions are to be ran

        Returns:

            obj: self
        """
        tools_to_run = []

        if dynamics:
            tools_to_run.append(constants.TOOL_DYNAMINE)
        if aggregation:
            tools_to_run.append(constants.TOOL_AGMATA)
        if early_folding_propensity:
            tools_to_run.append(constants.TOOL_EFOLDMINE)
        if disorder:
            tools_to_run.append(constants.TOOL_DISOMINE)

        return self.predict(tools=tools_to_run)

    def explicit_definition_predictions(self, backbone_dynamics=False,
                                        sidechain_dynamics=False,
                                        propoline_II=False,
                                        disorder_propensity=False,
                                        coil=False,
                                        beta_sheet=False,
                                        alpha_helix=False,
                                        early_folding_propensity=False,
                                        aggregation_propensity=False):

        """

        Instead of using a list of tools as in predict method, this one uses flags on the (explicit) type of
        predictions desired to create a list of tools and call the predict method.

        Parameters:

            backbone_dynamics (bool, optional) : Whether or not backbone dynamics are to be predicted
            sidechain_dynamics (bool, optional) : Whether or not sidechain dynamics are to be predicted
            propoline_II (bool, optional) : Whether or not propoline II propensity is to be predicted
            disorder_propensity (bool, optional) : Whether or not disorder propensity is to be predicted
            coil (bool, optional) : Whether or not coil propensity is to be predicted
            beta_sheet (bool, optional) : Whether or not beta sheet propensity is to be predicted
            alpha_helix (bool, optional) : Whether or not alpha helix propensity is to be predicted
            early_folding_propensity (bool, optional) : Whether or early folding propensity is to be predicted
            aggregation_propensity (bool, optional) : Whether or aggregation propensity is to be predicted

        Returns:

            obj: self

        """

        tools_to_run = []

        if backbone_dynamics or sidechain_dynamics or coil or beta_sheet or alpha_helix or propoline_II:
            tools_to_run.append(constants.TOOL_DYNAMINE)
        if disorder_propensity:
            tools_to_run.append(constants.TOOL_DISOMINE)
        if early_folding_propensity:
            tools_to_run.append(constants.TOOL_EFOLDMINE)
        if aggregation_propensity:
            tools_to_run.append(constants.TOOL_AGMATA)

        return self.predict(tools=tools_to_run)

    def _all_predictions(self):
        result = {}

        for tool in self.__predictor_runners__:
            result.update(self.__predictor_runners__[tool].allPredictions)

        return result

    def get_all_predictions_json(self, identifier):
        """
        Outputs all available predictions in a JSON formatted string. This still needs to be written in the desired
        output channel by the user.

        Parameters:

            identifier (str) : Identifier used as the root key of the JSON output.

        Returns:

            str : JSON string with outputs
        """
        self.__mineSuite__.allPredictions = self._all_predictions()

        return self.__mineSuite__.getAllPredictionsJson(identifier=identifier)

    def get_all_predictions(self, sequence_key=None):
        """
        Returns the values in dictionary form. It also allows to select the outputs of a single sequence from the
        original fasta file instead of all of them at once.

        Parameters:

            sequence_key (str, optional) : Sequence identifier specified as the FASTA header in the input
        file. It allow the user to select the output of a single sequence.

        Returns:

            reorganized_results (dict) : Dictionary which contains the output of the predictions.

        """
        result = self._all_predictions()

        if sequence_key is not None:
            results_to_reorganize = {sequence_key: result[sequence_key]}
        else:
            results_to_reorganize = result

        reorganized_results = self._organize_predictions_in_dictionary(results_to_reorganize)

        return reorganized_results

    def _organize_predictions_in_dictionary(self, results):
        new_result = {}
        for sequence in results:
            new_keys = {}

            for i, predictions in enumerate(results[sequence]):
                if i == 0:
                    new_keys['seq'] = [position[0] for position in results[sequence][predictions]]
                new_keys[predictions] = [position[1] for position in results[sequence][predictions]]

            new_result[sequence] = new_keys

        return new_result
